<?php
// $id:

/**
 * Function is utilized to retrieve the current translations for a node
 *
 * @param $node
 *      The node which we are retrieving the current translations for
 */
function node_translation_current_translations($node) {
  global $user;
  // retrieve the list of translatable languages the translator understands
  $languages = call_user_func('translation_framework_languages', TRUE);
  $translate_to = $translate_language = $current_translations = array(); // array of translations we can translate the node into

  // Get all of the available languages supported by the translation framework API
  foreach ($languages as $val => $language) {
    if (!$node->language) {
      // We want to have the node language set before we try and translate from that particular node
      $translate_language[$val] = l(t('Set node language to !language', array('!language' => $language)), 'node_translation/'. $node->nid .'/language/'. $val, array('attributes' => array('class' => 'node-translation-link-language')));
    }
    else {
      // Node has a language set hence we display the pertinent languages we can translate
      // the node into based off the language support in the translation framework API
      $translate_to[$val] = l(t('Translate into !language', array('!language' => $language)), 'node_translation/translate/'. $node->nid .'/'. $val, array('attributes' => array('class' => 'node-translation-link-translate')));
    }
  }

  // check if translations are available so we can show them
  $translations = translation_node_get_translations($node->tnid);
  if (!empty($translations)) {
    $title = '<h3>'. t('Current Translations') .'</h3>';
    $output = '';
    foreach ($translations as $translated_node) {
      if (isset($translated_node->language) && !is_null($translated_node->language) && drupal_strlen($translated_node->language) > 0 && isset($translate_to[$translated_node->language])) {
        // Remove the language from the list of available translations since we already have it translated
        unset($translate_to[$translated_node->language]);
      }
      if (!$node->language) {
        unset($translate_language[$translated_node->language]); // removing from the language we can set this node too
      }
      // Remove the current node from the list of translations we do not want to show it
      if ($translated_node->nid == $node->nid) { continue; }
      if ($translated_node->language) {
        // Only display this information if the language is one we actually support in the system
        if (isset($languages[$translated_node->language])) {
          $output .= '<div class="node_translation_available">'. l($languages[$translated_node->language] .': '. $translated_node->title, 'node/'. $translated_node->nid) .'</div>';
        }
      }
      else {
        $output .= '<div class="node_translation_available">'. l(t('No Language Specified: !title', array('!title' => $translated_node->title)), 'node/'. $translated_node->nid) .'</div>';
      }
    }
    $current_translations = array('title' => $title, 'body' => $output);
  }
  else {
    if ($node->language) {
      unset($translate_to[$node->language]);
    }
  }

  $language_neutral = FALSE;
  // will only allow the users to set the language for the node if there are languages to set
  // as well as if the user actually has permissions to update the specific node or it wont be displayed
  if (!empty($translate_language) && node_access('update', $node, $user)) {
    $title = '<h3>'. t('Set Current Nodes Language') .'</h3>';
    $output = '<div class="node_translation_language">';
    foreach ($translate_language as $option) {
      $output .= '<div class="node_translation_lang">'. $option .'</div>';
    }
    $output .= '</div>';
    if (isset($current_translations) && !empty($current_translations)) {
      $data[] = $current_translations;
    }
    $data[] = array('title' => $title, 'body' => $output);
    $language_neutral = TRUE;
  }

  // if we have more languages that we can display then show the form translate portion
  if (!empty($translate_to) && $language_neutral === FALSE) {
    $title = '<h3>'. t('Languages to translate into') .'</h3>';
    $output = '<div class="node_translation_language">';
    foreach ($translate_to as $option) {
      $output .= '<div class="node_translation_lang">'. $option .'</div>';
    }
    $output .= '</div>';
    if (isset($current_translations) && !empty($current_translations)) {
      $data[] = $current_translations;
    }
    $data[] = array('title' => $title, 'body' => $output);
  }
  _node_translation_set_headers();
  if (!isset($data)) {
    $data[] = array('title' => t('No Information Available'), 'body' => t('No translation information could be found regarding the node'));
  }
  print drupal_json($data);
  exit;
}

/**
 * Function performs the translations and will display it side by side for the users
 *
 * @param $node
 *      The node we are performing the translation on
 * @param $dst
 *      The language we wish to translate the node into
 */
function node_translation_translate($node, $dst) {
  _node_translation_set_headers();
  // retrieving the list of languages that are supported by our translation api
  $trans = call_user_func('translation_framework_languages', TRUE);
  // setting the title and body since if no translation we will display the original text
  $title = $node->title;
  $body = $node->body;

  // hooking into the translation framework and determining if it is working or not
  $func = 'translation_framework_translate_send';
  if (function_exists($func)) {
    $src = ($node->translation_source->language) ? $node->translation_source->language : variable_get('node_translation_source_language_default', 'en');
    $title = call_user_func($func, $node->title, $src, $dst);
    $body = call_user_func($func, $node->body, $src, $dst);
  }
  // the header for the table that we will display
  $header[] = array('data' => t('Element'), 'field' => 'element', 'sort' => 'desc');
  $header[] = array('data' => t('Source'), 'field' => 'source');
  $header[] = array('data' => t('Translation'), 'field' => 'translation');

  // creating the output that will be displayed to the user on screen
  $rows[] = array('element' => t('Title'), 'source' => $node->title, 'translation' => $title);
  $rows[] = array('element' => t('Body'), 'source' => $node->body, 'translation' => $body);

  // creating the output we will display
  $display_title = '<h3>'. t('Translation from !source to !translation', array('!source' => $trans[$node->language], '!translation' => $trans[$dst])) .'</h3>';
  $display_body = theme('table', $header, $rows);
  $display_footer = '<div style="text-align: center;">'. l(t('Create Translation'), 'node/add/'. $node->type, array('query' => array('translation' => $node->nid, 'language' => $dst))) .'</div>';
  // Stripping any excess whitespace so we are guaranteed to have correct data
  $display_body = preg_replace('/\s\s+/', ' ', $display_body);
  $display_body = str_replace(array("< ", " >"), array("<", ">"), $display_body);
  $data = array('title' => $display_title, 'body' => $display_body, 'footer' => $display_footer);
  print drupal_json($data);
  exit;
}

/**
 * Function sets a nodes language translation i.e. if the node was created originally without any language
 *
 * @param $node
 *      The node whose's language we are updating
 * @param $language
 *      The language we are setting the node too
 */
function node_translation_language($node, $language = NULL) {
  global $user;

  // checking if the user has the pertinent access to update the node
  if (node_access('update', $node, $user)) {
    if (!is_null($language) && drupal_strlen($language) > 0) {
      $node->language = $language;
      node_save($node);
    }
  }
  else {
    $data = array('title' => t('User does not have node update priviledges'));
    _node_translation_set_headers();
    print drupal_json($data);
    exit;
  }
  // reloading the available translations for the node
  node_translation_current_translations($node);
}

/**
 * Function to set the pertinent HTTP headers for the data we are returning
 */
function _node_translation_set_headers() {
  drupal_set_header('Content-Encoding: UTF-8');
  drupal_set_header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
  drupal_set_header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
  drupal_set_header("Cache-Control: no-store, no-cache, must-revalidate");
  drupal_set_header("Cache-Control: post-check=0, pre-check=0", false);
  drupal_set_header("Pragma: no-cache");
}

